// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_
#define COMPONENTS_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "components/payments/payment_app.mojom-shared-internal.h"


namespace payments {
namespace mojom {
class PaymentAppOptionDataView;

class PaymentAppManifestDataView;



}  // namespace mojom
}  // namespace payments

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::payments::mojom::PaymentAppOptionDataView> {
  using Data = ::payments::mojom::internal::PaymentAppOption_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::payments::mojom::PaymentAppManifestDataView> {
  using Data = ::payments::mojom::internal::PaymentAppManifest_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace payments {
namespace mojom {

enum class PaymentAppManifestError : int32_t {
  NONE,
  NOT_IMPLEMENTED,
};

inline std::ostream& operator<<(std::ostream& os, PaymentAppManifestError value) {
  switch(value) {
    case PaymentAppManifestError::NONE:
      return os << "PaymentAppManifestError::NONE";
    case PaymentAppManifestError::NOT_IMPLEMENTED:
      return os << "PaymentAppManifestError::NOT_IMPLEMENTED";
    default:
      return os << "Unknown PaymentAppManifestError value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PaymentAppManifestError value) {
  return internal::PaymentAppManifestError_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class PaymentAppManagerInterfaceBase {};

using PaymentAppManagerPtrDataView =
    mojo::InterfacePtrDataView<PaymentAppManagerInterfaceBase>;
using PaymentAppManagerRequestDataView =
    mojo::InterfaceRequestDataView<PaymentAppManagerInterfaceBase>;
using PaymentAppManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PaymentAppManagerInterfaceBase>;
using PaymentAppManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PaymentAppManagerInterfaceBase>;
class PaymentAppOptionDataView {
 public:
  PaymentAppOptionDataView() {}

  PaymentAppOptionDataView(
      internal::PaymentAppOption_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLabelDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLabel(UserType* output) {
    auto* pointer = data_->label.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIcon(UserType* output) {
    auto* pointer = data_->icon.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetEnabledMethodsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEnabledMethods(UserType* output) {
    auto* pointer = data_->enabled_methods.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
 private:
  internal::PaymentAppOption_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentAppManifestDataView {
 public:
  PaymentAppManifestDataView() {}

  PaymentAppManifestDataView(
      internal::PaymentAppManifest_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetLabelDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLabel(UserType* output) {
    auto* pointer = data_->label.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIcon(UserType* output) {
    auto* pointer = data_->icon.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetOptionsDataView(
      mojo::ArrayDataView<PaymentAppOptionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptions(UserType* output) {
    auto* pointer = data_->options.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::payments::mojom::PaymentAppOptionDataView>>(
        pointer, output, context_);
  }
 private:
  internal::PaymentAppManifest_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentAppManager_SetManifest_ParamsDataView {
 public:
  PaymentAppManager_SetManifest_ParamsDataView() {}

  PaymentAppManager_SetManifest_ParamsDataView(
      internal::PaymentAppManager_SetManifest_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetServiceWorkerScopeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadServiceWorkerScope(UserType* output) {
    auto* pointer = data_->service_worker_scope.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetPaymentAppManifestDataView(
      PaymentAppManifestDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPaymentAppManifest(UserType* output) {
    auto* pointer = data_->payment_app_manifest.Get();
    return mojo::internal::Deserialize<::payments::mojom::PaymentAppManifestDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentAppManager_SetManifest_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentAppManager_SetManifest_ResponseParamsDataView {
 public:
  PaymentAppManager_SetManifest_ResponseParamsDataView() {}

  PaymentAppManager_SetManifest_ResponseParamsDataView(
      internal::PaymentAppManager_SetManifest_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadError(UserType* output) const {
    auto data_value = data_->error;
    return mojo::internal::Deserialize<::payments::mojom::PaymentAppManifestError>(
        data_value, output);
  }

  PaymentAppManifestError error() const {
    return static_cast<PaymentAppManifestError>(data_->error);
  }
 private:
  internal::PaymentAppManager_SetManifest_ResponseParams_Data* data_ = nullptr;
};



}  // namespace mojom
}  // namespace payments

namespace std {

template <>
struct hash<::payments::mojom::PaymentAppManifestError>
    : public mojo::internal::EnumHashImpl<::payments::mojom::PaymentAppManifestError> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::payments::mojom::PaymentAppManifestError, ::payments::mojom::PaymentAppManifestError> {
  static ::payments::mojom::PaymentAppManifestError ToMojom(::payments::mojom::PaymentAppManifestError input) { return input; }
  static bool FromMojom(::payments::mojom::PaymentAppManifestError input, ::payments::mojom::PaymentAppManifestError* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentAppManifestError, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::payments::mojom::PaymentAppManifestError, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::payments::mojom::PaymentAppManifestError>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentAppOptionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::PaymentAppOptionDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::PaymentAppOption_Data);
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_label, context);
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_icon, context);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_id, context);
    decltype(CallWithContext(Traits::enabled_methods, input, custom_context)) in_enabled_methods = CallWithContext(Traits::enabled_methods, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_enabled_methods, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::PaymentAppOption_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::PaymentAppOption_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    typename decltype(result->label)::BaseType* label_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_label, buffer, &label_ptr, context);
    result->label.Set(label_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->label.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null label in PaymentAppOption struct");
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    typename decltype(result->icon)::BaseType* icon_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_icon, buffer, &icon_ptr, context);
    result->icon.Set(icon_ptr);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype(result->id)::BaseType* id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_ptr, context);
    result->id.Set(id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in PaymentAppOption struct");
    decltype(CallWithContext(Traits::enabled_methods, input, custom_context)) in_enabled_methods = CallWithContext(Traits::enabled_methods, input, custom_context);
    typename decltype(result->enabled_methods)::BaseType* enabled_methods_ptr;
    const mojo::internal::ContainerValidateParams enabled_methods_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_enabled_methods, buffer, &enabled_methods_ptr, &enabled_methods_validate_params,
        context);
    result->enabled_methods.Set(enabled_methods_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->enabled_methods.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null enabled_methods in PaymentAppOption struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::PaymentAppOption_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::PaymentAppOptionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentAppManifestDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::PaymentAppManifestDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::PaymentAppManifest_Data);
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_label, context);
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_icon, context);
    decltype(CallWithContext(Traits::options, input, custom_context)) in_options = CallWithContext(Traits::options, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::payments::mojom::PaymentAppOptionDataView>>(
        in_options, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::PaymentAppManifest_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::PaymentAppManifest_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::label, input, custom_context)) in_label = CallWithContext(Traits::label, input, custom_context);
    typename decltype(result->label)::BaseType* label_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_label, buffer, &label_ptr, context);
    result->label.Set(label_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->label.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null label in PaymentAppManifest struct");
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    typename decltype(result->icon)::BaseType* icon_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_icon, buffer, &icon_ptr, context);
    result->icon.Set(icon_ptr);
    decltype(CallWithContext(Traits::options, input, custom_context)) in_options = CallWithContext(Traits::options, input, custom_context);
    typename decltype(result->options)::BaseType* options_ptr;
    const mojo::internal::ContainerValidateParams options_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::payments::mojom::PaymentAppOptionDataView>>(
        in_options, buffer, &options_ptr, &options_validate_params,
        context);
    result->options.Set(options_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->options.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null options in PaymentAppManifest struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::PaymentAppManifest_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::PaymentAppManifestDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace payments {
namespace mojom {

inline void PaymentAppOptionDataView::GetLabelDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->label.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppOptionDataView::GetIconDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->icon.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppOptionDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppOptionDataView::GetEnabledMethodsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->enabled_methods.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}


inline void PaymentAppManifestDataView::GetLabelDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->label.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppManifestDataView::GetIconDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->icon.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppManifestDataView::GetOptionsDataView(
    mojo::ArrayDataView<PaymentAppOptionDataView>* output) {
  auto pointer = data_->options.Get();
  *output = mojo::ArrayDataView<PaymentAppOptionDataView>(pointer, context_);
}


inline void PaymentAppManager_SetManifest_ParamsDataView::GetServiceWorkerScopeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->service_worker_scope.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentAppManager_SetManifest_ParamsDataView::GetPaymentAppManifestDataView(
    PaymentAppManifestDataView* output) {
  auto pointer = data_->payment_app_manifest.Get();
  *output = PaymentAppManifestDataView(pointer, context_);
}





}  // namespace mojom
}  // namespace payments

#endif  // COMPONENTS_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_
