// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_GPU_MEMORY_BUFFER_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_GPU_MEMORY_BUFFER_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/gpu_memory_buffer.mojom-shared.h"
#include "ui/gfx/mojo/buffer_types.mojom.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "gpu/config/dx_diag_node.h"
#include "base/values.h"
#include "cc/output/compositor_frame.h"
#include "third_party/skia/include/core/SkImageFilter.h"
#include "gpu/command_buffer/common/sync_token.h"
#include "cc/output/filter_operation.h"
#include "ui/display/display.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "ui/gfx/icc_profile.h"
#include "ui/gfx/geometry/insets.h"
#include "base/version.h"
#include "gpu/config/gpu_info.h"
#include "base/time/time.h"
#include "ui/gfx/gpu_memory_buffer.h"
#include "cc/output/filter_operations.h"
#include "base/strings/string16.h"
#include "ui/gfx/selection_bound.h"
#include "base/unguessable_token.h"
#include "ui/gfx/geometry/size.h"
#include "ui/gfx/geometry/point.h"
#include "ui/events/latency_info.h"
#include "cc/output/begin_frame_args.h"
#include "cc/surfaces/surface_sequence.h"
#include "cc/output/compositor_frame_metadata.h"
#include "ui/events/event.h"
#include "cc/quads/draw_quad.h"
#include "ui/gfx/native_pixmap_handle.h"
#include "ui/gfx/geometry/rect_f.h"
#include "cc/quads/render_pass.h"
#include "gpu/command_buffer/common/mailbox_holder.h"
#include "cc/resources/returned_resource.h"
#include "base/files/file_path.h"
#include "cc/surfaces/surface_id.h"
#include "ui/gfx/geometry/point_f.h"
#include "gpu/command_buffer/common/mailbox.h"
#include "cc/input/selection.h"
#include "third_party/skia/include/core/SkBitmap.h"
#include "ui/gfx/buffer_types.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "cc/surfaces/frame_sink_id.h"
#include "ui/gfx/native_widget_types.h"
#include "cc/quads/shared_quad_state.h"
#include "cc/resources/transferable_resource.h"
#include "ui/gfx/transform.h"
#include "cc/surfaces/local_frame_id.h"
#include "ui/gfx/geometry/rect.h"
#include "gpu/command_buffer/common/capabilities.h"
#include "gpu/ipc/common/surface_handle.h"
#include "ui/gfx/geometry/vector2d.h"
#include "cc/quads/render_pass_id.h"


namespace ui {
namespace mojom {
class NativePixmapHandle;
using NativePixmapHandlePtr = mojo::StructPtr<NativePixmapHandle>;

class GpuMemoryBufferHandle;
using GpuMemoryBufferHandlePtr = mojo::StructPtr<GpuMemoryBufferHandle>;







class  NativePixmapHandle {
 public:
  using DataView = NativePixmapHandleDataView;
  using Data_ = internal::NativePixmapHandle_Data;

  static NativePixmapHandlePtr New();

  template <typename U>
  static NativePixmapHandlePtr From(const U& u) {
    return mojo::TypeConverter<NativePixmapHandlePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, NativePixmapHandle>::Convert(*this);
  }

  NativePixmapHandle();
  ~NativePixmapHandle();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = NativePixmapHandlePtr>
  NativePixmapHandlePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, NativePixmapHandle>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        NativePixmapHandle::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        NativePixmapHandle::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  mojo::ScopedHandle fd;
  int32_t stride;
 private:
  DISALLOW_COPY_AND_ASSIGN(NativePixmapHandle);
};



class  GpuMemoryBufferHandle {
 public:
  using DataView = GpuMemoryBufferHandleDataView;
  using Data_ = internal::GpuMemoryBufferHandle_Data;

  static GpuMemoryBufferHandlePtr New();

  template <typename U>
  static GpuMemoryBufferHandlePtr From(const U& u) {
    return mojo::TypeConverter<GpuMemoryBufferHandlePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, GpuMemoryBufferHandle>::Convert(*this);
  }

  GpuMemoryBufferHandle();
  ~GpuMemoryBufferHandle();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = GpuMemoryBufferHandlePtr>
  GpuMemoryBufferHandlePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, GpuMemoryBufferHandle>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        GpuMemoryBufferHandle::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        GpuMemoryBufferHandle::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  gfx::GpuMemoryBufferType type;
  gfx::GpuMemoryBufferId id;
  mojo::ScopedHandle buffer_handle;
  uint32_t offset;
  int32_t stride;
  NativePixmapHandlePtr native_pixmap_handle;
 private:
  DISALLOW_COPY_AND_ASSIGN(GpuMemoryBufferHandle);
};


template <typename StructPtrType>
NativePixmapHandlePtr NativePixmapHandle::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->fd = mojo::internal::Clone(fd);
  rv->stride = mojo::internal::Clone(stride);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, NativePixmapHandle>::value>::type*>
bool NativePixmapHandle::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->fd, other.fd))
    return false;
  if (!mojo::internal::Equals(this->stride, other.stride))
    return false;
  return true;
}
template <typename StructPtrType>
GpuMemoryBufferHandlePtr GpuMemoryBufferHandle::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->type = mojo::internal::Clone(type);
  rv->id = mojo::internal::Clone(id);
  rv->buffer_handle = mojo::internal::Clone(buffer_handle);
  rv->offset = mojo::internal::Clone(offset);
  rv->stride = mojo::internal::Clone(stride);
  rv->native_pixmap_handle = mojo::internal::Clone(native_pixmap_handle);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, GpuMemoryBufferHandle>::value>::type*>
bool GpuMemoryBufferHandle::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->type, other.type))
    return false;
  if (!mojo::internal::Equals(this->id, other.id))
    return false;
  if (!mojo::internal::Equals(this->buffer_handle, other.buffer_handle))
    return false;
  if (!mojo::internal::Equals(this->offset, other.offset))
    return false;
  if (!mojo::internal::Equals(this->stride, other.stride))
    return false;
  if (!mojo::internal::Equals(this->native_pixmap_handle, other.native_pixmap_handle))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ui

namespace mojo {


template <>
struct  StructTraits<::ui::mojom::NativePixmapHandle::DataView,
                                         ::ui::mojom::NativePixmapHandlePtr> {
  static bool IsNull(const ::ui::mojom::NativePixmapHandlePtr& input) { return !input; }
  static void SetToNull(::ui::mojom::NativePixmapHandlePtr* output) { output->reset(); }

  static  decltype(::ui::mojom::NativePixmapHandle::fd)& fd(
       ::ui::mojom::NativePixmapHandlePtr& input) {
    return input->fd;
  }

  static decltype(::ui::mojom::NativePixmapHandle::stride) stride(
      const ::ui::mojom::NativePixmapHandlePtr& input) {
    return input->stride;
  }

  static bool Read(::ui::mojom::NativePixmapHandle::DataView input, ::ui::mojom::NativePixmapHandlePtr* output);
};


template <>
struct  StructTraits<::ui::mojom::GpuMemoryBufferHandle::DataView,
                                         ::ui::mojom::GpuMemoryBufferHandlePtr> {
  static bool IsNull(const ::ui::mojom::GpuMemoryBufferHandlePtr& input) { return !input; }
  static void SetToNull(::ui::mojom::GpuMemoryBufferHandlePtr* output) { output->reset(); }

  static decltype(::ui::mojom::GpuMemoryBufferHandle::type) type(
      const ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->type;
  }

  static const decltype(::ui::mojom::GpuMemoryBufferHandle::id)& id(
      const ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->id;
  }

  static  decltype(::ui::mojom::GpuMemoryBufferHandle::buffer_handle)& buffer_handle(
       ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->buffer_handle;
  }

  static decltype(::ui::mojom::GpuMemoryBufferHandle::offset) offset(
      const ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->offset;
  }

  static decltype(::ui::mojom::GpuMemoryBufferHandle::stride) stride(
      const ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->stride;
  }

  static  decltype(::ui::mojom::GpuMemoryBufferHandle::native_pixmap_handle)& native_pixmap_handle(
       ::ui::mojom::GpuMemoryBufferHandlePtr& input) {
    return input->native_pixmap_handle;
  }

  static bool Read(::ui::mojom::GpuMemoryBufferHandle::DataView input, ::ui::mojom::GpuMemoryBufferHandlePtr* output);
};

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_GPU_MEMORY_BUFFER_MOJOM_H_