// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/presentation/presentation.mojom-shared.h"
#include "url/mojo/url.mojom.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "base/values.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "base/strings/string16.h"
#include "base/version.h"
#include "base/time/time.h"
#include "third_party/WebKit/public/web/WebWindowFeatures.h"
#include "ui/gfx/geometry/insets.h"
#include "content/public/common/media_metadata.h"
#include "content/common/bluetooth/web_bluetooth_device_id.h"
#include "ui/gfx/geometry/size.h"
#include "content/public/common/referrer.h"
#include "ui/gfx/geometry/point.h"
#include "ui/gfx/geometry/rect_f.h"
#include "base/files/file_path.h"
#include "url/origin.h"
#include "ui/gfx/geometry/point_f.h"
#include "base/unguessable_token.h"
#include "third_party/WebKit/public/platform/WebReferrerPolicy.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "url/gurl.h"
#include "device/bluetooth/bluetooth_uuid.h"
#include "ui/gfx/geometry/rect.h"
#include "ui/gfx/geometry/vector2d.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class PresentationService;
using PresentationServicePtr = mojo::InterfacePtr<PresentationService>;
using PresentationServicePtrInfo = mojo::InterfacePtrInfo<PresentationService>;
using ThreadSafePresentationServicePtr =
    mojo::ThreadSafeInterfacePtr<PresentationService>;
using PresentationServiceRequest = mojo::InterfaceRequest<PresentationService>;
using PresentationServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<PresentationService>;
using ThreadSafePresentationServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PresentationService>;
using PresentationServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PresentationService>;
using PresentationServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PresentationService>;

class PresentationServiceClient;
using PresentationServiceClientPtr = mojo::InterfacePtr<PresentationServiceClient>;
using PresentationServiceClientPtrInfo = mojo::InterfacePtrInfo<PresentationServiceClient>;
using ThreadSafePresentationServiceClientPtr =
    mojo::ThreadSafeInterfacePtr<PresentationServiceClient>;
using PresentationServiceClientRequest = mojo::InterfaceRequest<PresentationServiceClient>;
using PresentationServiceClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<PresentationServiceClient>;
using ThreadSafePresentationServiceClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<PresentationServiceClient>;
using PresentationServiceClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<PresentationServiceClient>;
using PresentationServiceClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<PresentationServiceClient>;

class PresentationSessionInfo;
using PresentationSessionInfoPtr = mojo::StructPtr<PresentationSessionInfo>;

class PresentationError;
using PresentationErrorPtr = mojo::InlinedStructPtr<PresentationError>;

class SessionMessage;
using SessionMessagePtr = mojo::StructPtr<SessionMessage>;


class PresentationServiceProxy;

template <typename ImplRefTraits>
class PresentationServiceStub;

class PresentationServiceRequestValidator;
class PresentationServiceResponseValidator;

class CONTENT_EXPORT PresentationService
    : public PresentationServiceInterfaceBase {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = PresentationServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = PresentationServiceStub<ImplRefTraits>;

  using RequestValidator_ = PresentationServiceRequestValidator;
  using ResponseValidator_ = PresentationServiceResponseValidator;
  virtual ~PresentationService() {}

  virtual void SetClient(PresentationServiceClientPtr client) = 0;

  virtual void SetDefaultPresentationUrls(const std::vector<GURL>& presentation_urls) = 0;

  virtual void ListenForScreenAvailability(const GURL& availability_url) = 0;

  virtual void StopListeningForScreenAvailability(const GURL& availability_url) = 0;


  using StartSessionCallback = base::Callback<void(PresentationSessionInfoPtr, PresentationErrorPtr)>;
  virtual void StartSession(const std::vector<GURL>& presentation_urls, const StartSessionCallback& callback) = 0;


  using JoinSessionCallback = base::Callback<void(PresentationSessionInfoPtr, PresentationErrorPtr)>;
  virtual void JoinSession(const std::vector<GURL>& presentation_urls, const base::Optional<std::string>& presentation_id, const JoinSessionCallback& callback) = 0;


  using SendSessionMessageCallback = base::Callback<void(bool)>;
  virtual void SendSessionMessage(PresentationSessionInfoPtr sessionInfo, SessionMessagePtr message_request, const SendSessionMessageCallback& callback) = 0;

  virtual void CloseConnection(const GURL& presentation_url, const std::string& presentation_id) = 0;

  virtual void Terminate(const GURL& presentation_url, const std::string& presentation_id) = 0;

  virtual void ListenForSessionMessages(PresentationSessionInfoPtr sessionInfo) = 0;
};

class PresentationServiceClientProxy;

template <typename ImplRefTraits>
class PresentationServiceClientStub;

class PresentationServiceClientRequestValidator;

class CONTENT_EXPORT PresentationServiceClient
    : public PresentationServiceClientInterfaceBase {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = PresentationServiceClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = PresentationServiceClientStub<ImplRefTraits>;

  using RequestValidator_ = PresentationServiceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  virtual ~PresentationServiceClient() {}

  virtual void OnScreenAvailabilityNotSupported(const GURL& url) = 0;

  virtual void OnScreenAvailabilityUpdated(const GURL& url, bool available) = 0;

  virtual void OnConnectionStateChanged(PresentationSessionInfoPtr connection, PresentationConnectionState newState) = 0;

  virtual void OnConnectionClosed(PresentationSessionInfoPtr connection, PresentationConnectionCloseReason reason, const std::string& message) = 0;

  virtual void OnSessionMessagesReceived(PresentationSessionInfoPtr sessionInfo, std::vector<SessionMessagePtr> messages) = 0;

  virtual void OnDefaultSessionStarted(PresentationSessionInfoPtr sessionInfo) = 0;

  virtual void OnReceiverConnectionAvailable(PresentationSessionInfoPtr sessionInfo) = 0;
};

class CONTENT_EXPORT PresentationServiceProxy
    : public PresentationService {
 public:
  explicit PresentationServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetClient(PresentationServiceClientPtr client) override;
  void SetDefaultPresentationUrls(const std::vector<GURL>& presentation_urls) override;
  void ListenForScreenAvailability(const GURL& availability_url) override;
  void StopListeningForScreenAvailability(const GURL& availability_url) override;
  void StartSession(const std::vector<GURL>& presentation_urls, const StartSessionCallback& callback) override;
  void JoinSession(const std::vector<GURL>& presentation_urls, const base::Optional<std::string>& presentation_id, const JoinSessionCallback& callback) override;
  void SendSessionMessage(PresentationSessionInfoPtr sessionInfo, SessionMessagePtr message_request, const SendSessionMessageCallback& callback) override;
  void CloseConnection(const GURL& presentation_url, const std::string& presentation_id) override;
  void Terminate(const GURL& presentation_url, const std::string& presentation_id) override;
  void ListenForSessionMessages(PresentationSessionInfoPtr sessionInfo) override;

  void set_group_controller(
        scoped_refptr<mojo::AssociatedGroupController> group_controller) {
    group_controller_ = std::move(group_controller);
  }
 private:
  mojo::MessageReceiverWithResponder* receiver_;
  scoped_refptr<mojo::AssociatedGroupController> group_controller_;
};

class CONTENT_EXPORT PresentationServiceClientProxy
    : public PresentationServiceClient {
 public:
  explicit PresentationServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnScreenAvailabilityNotSupported(const GURL& url) override;
  void OnScreenAvailabilityUpdated(const GURL& url, bool available) override;
  void OnConnectionStateChanged(PresentationSessionInfoPtr connection, PresentationConnectionState newState) override;
  void OnConnectionClosed(PresentationSessionInfoPtr connection, PresentationConnectionCloseReason reason, const std::string& message) override;
  void OnSessionMessagesReceived(PresentationSessionInfoPtr sessionInfo, std::vector<SessionMessagePtr> messages) override;
  void OnDefaultSessionStarted(PresentationSessionInfoPtr sessionInfo) override;
  void OnReceiverConnectionAvailable(PresentationSessionInfoPtr sessionInfo) override;

  void set_group_controller(
        scoped_refptr<mojo::AssociatedGroupController> group_controller) {
    group_controller_ = std::move(group_controller);
  }
 private:
  mojo::MessageReceiverWithResponder* receiver_;
  scoped_refptr<mojo::AssociatedGroupController> group_controller_;
};
class CONTENT_EXPORT PresentationServiceStubDispatch {
 public:
  static bool Accept(PresentationService* impl,
                     mojo::internal::SerializationContext* context,
                     mojo::Message* message);
  static bool AcceptWithResponder(PresentationService* impl,
                                  mojo::internal::SerializationContext* context,
                                  mojo::Message* message,
                                  mojo::MessageReceiverWithStatus* responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PresentationService>>
class PresentationServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PresentationServiceStub() {}
  ~PresentationServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message,
        responder);
  }

 private:
  ImplPointerType sink_;
  mojo::internal::SerializationContext serialization_context_;
};
class CONTENT_EXPORT PresentationServiceClientStubDispatch {
 public:
  static bool Accept(PresentationServiceClient* impl,
                     mojo::internal::SerializationContext* context,
                     mojo::Message* message);
  static bool AcceptWithResponder(PresentationServiceClient* impl,
                                  mojo::internal::SerializationContext* context,
                                  mojo::Message* message,
                                  mojo::MessageReceiverWithStatus* responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PresentationServiceClient>>
class PresentationServiceClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PresentationServiceClientStub() {}
  ~PresentationServiceClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PresentationServiceClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message,
        responder);
  }

 private:
  ImplPointerType sink_;
  mojo::internal::SerializationContext serialization_context_;
};
class CONTENT_EXPORT PresentationServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationServiceClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT PresentationServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};



class CONTENT_EXPORT PresentationError {
 public:
  using DataView = PresentationErrorDataView;
  using Data_ = internal::PresentationError_Data;

  static PresentationErrorPtr New();

  template <typename U>
  static PresentationErrorPtr From(const U& u) {
    return mojo::TypeConverter<PresentationErrorPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationError>::Convert(*this);
  }

  PresentationError();
  ~PresentationError();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PresentationErrorPtr>
  PresentationErrorPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PresentationError>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PresentationError::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PresentationError::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  PresentationErrorType error_type;
  std::string message;
};






class CONTENT_EXPORT PresentationSessionInfo {
 public:
  using DataView = PresentationSessionInfoDataView;
  using Data_ = internal::PresentationSessionInfo_Data;

  static PresentationSessionInfoPtr New();

  template <typename U>
  static PresentationSessionInfoPtr From(const U& u) {
    return mojo::TypeConverter<PresentationSessionInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationSessionInfo>::Convert(*this);
  }

  PresentationSessionInfo();
  ~PresentationSessionInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PresentationSessionInfoPtr>
  PresentationSessionInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, PresentationSessionInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        PresentationSessionInfo::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        PresentationSessionInfo::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  GURL url;
  std::string id;
};




class CONTENT_EXPORT SessionMessage {
 public:
  using DataView = SessionMessageDataView;
  using Data_ = internal::SessionMessage_Data;

  static SessionMessagePtr New();

  template <typename U>
  static SessionMessagePtr From(const U& u) {
    return mojo::TypeConverter<SessionMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SessionMessage>::Convert(*this);
  }

  SessionMessage();
  ~SessionMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SessionMessagePtr>
  SessionMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SessionMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SessionMessage::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SessionMessage::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  PresentationMessageType type;
  base::Optional<std::string> message;
  base::Optional<std::vector<uint8_t>> data;
};


template <typename StructPtrType>
PresentationSessionInfoPtr PresentationSessionInfo::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->url = mojo::internal::Clone(url);
  rv->id = mojo::internal::Clone(id);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PresentationSessionInfo>::value>::type*>
bool PresentationSessionInfo::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->url, other.url))
    return false;
  if (!mojo::internal::Equals(this->id, other.id))
    return false;
  return true;
}
template <typename StructPtrType>
PresentationErrorPtr PresentationError::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->error_type = mojo::internal::Clone(error_type);
  rv->message = mojo::internal::Clone(message);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, PresentationError>::value>::type*>
bool PresentationError::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->error_type, other.error_type))
    return false;
  if (!mojo::internal::Equals(this->message, other.message))
    return false;
  return true;
}
template <typename StructPtrType>
SessionMessagePtr SessionMessage::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->type = mojo::internal::Clone(type);
  rv->message = mojo::internal::Clone(message);
  rv->data = mojo::internal::Clone(data);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SessionMessage>::value>::type*>
bool SessionMessage::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->type, other.type))
    return false;
  if (!mojo::internal::Equals(this->message, other.message))
    return false;
  if (!mojo::internal::Equals(this->data, other.data))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::PresentationSessionInfo::DataView,
                                         ::blink::mojom::PresentationSessionInfoPtr> {
  static bool IsNull(const ::blink::mojom::PresentationSessionInfoPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::PresentationSessionInfoPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::PresentationSessionInfo::url)& url(
      const ::blink::mojom::PresentationSessionInfoPtr& input) {
    return input->url;
  }

  static const decltype(::blink::mojom::PresentationSessionInfo::id)& id(
      const ::blink::mojom::PresentationSessionInfoPtr& input) {
    return input->id;
  }

  static bool Read(::blink::mojom::PresentationSessionInfo::DataView input, ::blink::mojom::PresentationSessionInfoPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::PresentationError::DataView,
                                         ::blink::mojom::PresentationErrorPtr> {
  static bool IsNull(const ::blink::mojom::PresentationErrorPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::PresentationErrorPtr* output) { output->reset(); }

  static decltype(::blink::mojom::PresentationError::error_type) error_type(
      const ::blink::mojom::PresentationErrorPtr& input) {
    return input->error_type;
  }

  static const decltype(::blink::mojom::PresentationError::message)& message(
      const ::blink::mojom::PresentationErrorPtr& input) {
    return input->message;
  }

  static bool Read(::blink::mojom::PresentationError::DataView input, ::blink::mojom::PresentationErrorPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::SessionMessage::DataView,
                                         ::blink::mojom::SessionMessagePtr> {
  static bool IsNull(const ::blink::mojom::SessionMessagePtr& input) { return !input; }
  static void SetToNull(::blink::mojom::SessionMessagePtr* output) { output->reset(); }

  static decltype(::blink::mojom::SessionMessage::type) type(
      const ::blink::mojom::SessionMessagePtr& input) {
    return input->type;
  }

  static const decltype(::blink::mojom::SessionMessage::message)& message(
      const ::blink::mojom::SessionMessagePtr& input) {
    return input->message;
  }

  static const decltype(::blink::mojom::SessionMessage::data)& data(
      const ::blink::mojom::SessionMessagePtr& input) {
    return input->data;
  }

  static bool Read(::blink::mojom::SessionMessage::DataView input, ::blink::mojom::SessionMessagePtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PRESENTATION_PRESENTATION_MOJOM_H_